import PocketBase from 'pocketbase';

// Types for our data models
interface Product {
	id: string;
	name: string;
	description: string;
	price: number;
	stock: number;
	category: string;
	images: string[];
	variants?: Record<string, unknown>[];
	metadata?: Record<string, unknown>;
	createdAt: string;
	updatedAt: string;
}

interface Order {
	id: string;
	userId: string;
	items: OrderItem[];
	status: OrderStatus;
	total: number;
	shippingAddress: Address;
	billingAddress: Address;
	paymentInfo: PaymentInfo;
	createdAt: string;
	updatedAt: string;
}

interface OrderItem {
	productId: string;
	quantity: number;
	price: number;
	variants?: Record<string, unknown>;
}

interface User {
	id: string;
	email: string;
	name: string;
	addresses: Address[];
	orders?: string[];
	wishlist?: string[];
	metadata?: Record<string, unknown>;
}

interface Address {
	street: string;
	city: string;
	state: string;
	country: string;
	zipCode: string;
}

interface PaymentInfo {
	provider: string;
	transactionId: string;
	status: string;
	amount: number;
}

enum OrderStatus {
	PENDING = 'pending',
	PAID = 'paid',
	PROCESSING = 'processing',
	SHIPPED = 'shipped',
	DELIVERED = 'delivered',
	CANCELLED = 'cancelled',
	REFUNDED = 'refunded'
}

class WebshopAPI {
	private pb: PocketBase;

	constructor(url: string) {
		this.pb = new PocketBase(url);
	}

	// Authentication methods
	async login(email: string, password: string) {
		return await this.pb.collection('users').authWithPassword(email, password);
	}

	async register(userData: Partial<User>, password: string) {
		return await this.pb.collection('users').create({
			...userData,
			passwordConfirm: password,
			password
		});
	}

	async logout() {
		this.pb.authStore.clear();
	}

	async requestPasswordReset(email: string) {
		return await this.pb.collection('users').requestPasswordReset(email);
	}

	// Product methods
	async getProducts(page: number = 1, perPage: number = 20, filters?: string) {
		return await this.pb.collection('products').getList(page, perPage, {
			filter: filters,
			sort: '-created'
		});
	}

	async getProduct(id: string) {
		return await this.pb.collection('products').getOne<Product>(id);
	}

	async searchProducts(query: string, page: number = 1, perPage: number = 20) {
		return await this.pb.collection('products').getList(page, perPage, {
			filter: `name ~ "${query}" || description ~ "${query}"`
		});
	}

	async getProductsByCategory(category: string, page: number = 1, perPage: number = 20) {
		return await this.pb.collection('products').getList(page, perPage, {
			filter: `category = "${category}"`
		});
	}

	// Cart methods (using local storage for cart management)
	getCart(): OrderItem[] {
		const cart = localStorage.getItem('cart');
		return cart ? JSON.parse(cart) : [];
	}

	addToCart(item: OrderItem) {
		const cart = this.getCart();
		const existingItem = cart.find((i) => i.productId === item.productId);

		if (existingItem) {
			existingItem.quantity += item.quantity;
		} else {
			cart.push(item);
		}

		localStorage.setItem('cart', JSON.stringify(cart));
	}

	updateCartItem(productId: string, quantity: number) {
		const cart = this.getCart();
		const item = cart.find((i) => i.productId === productId);

		if (item) {
			item.quantity = quantity;
			localStorage.setItem('cart', JSON.stringify(cart));
		}
	}

	removeFromCart(productId: string) {
		const cart = this.getCart();
		const updatedCart = cart.filter((i) => i.productId !== productId);
		localStorage.setItem('cart', JSON.stringify(updatedCart));
	}

	clearCart() {
		localStorage.removeItem('cart');
	}

	// Order methods
	async createOrder(orderData: Partial<Order>) {
		return await this.pb.collection('orders').create(orderData);
	}

	async getOrder(id: string) {
		return await this.pb.collection('orders').getOne<Order>(id);
	}

	async getUserOrders(userId: string, page: number = 1, perPage: number = 20) {
		return await this.pb.collection('orders').getList(page, perPage, {
			filter: `userId = "${userId}"`,
			sort: '-created'
		});
	}

	async updateOrderStatus(orderId: string, status: OrderStatus) {
		return await this.pb.collection('orders').update(orderId, { status });
	}

	// User profile methods
	async getCurrentUser() {
		if (!this.pb.authStore.isValid) return null;
		return await this.pb.collection('users').getOne<User>(this.pb.authStore.model?.id as string);
	}

	async updateUserProfile(userId: string, userData: Partial<User>) {
		return await this.pb.collection('users').update(userId, userData);
	}

	async addAddress(userId: string, address: Address) {
		const user = await this.getCurrentUser();
		if (!user) throw new Error('User not authenticated');

		const addresses = [...(user.addresses || []), address];
		return await this.updateUserProfile(userId, { addresses });
	}

	// Wishlist methods
	async addToWishlist(userId: string, productId: string) {
		const user = await this.getCurrentUser();
		if (!user) throw new Error('User not authenticated');

		const wishlist = [...(user.wishlist || []), productId];
		return await this.updateUserProfile(userId, { wishlist });
	}

	async removeFromWishlist(userId: string, productId: string) {
		const user = await this.getCurrentUser();
		if (!user) throw new Error('User not authenticated');

		const wishlist = (user.wishlist || []).filter((id: unknown) => id !== productId);
		return await this.updateUserProfile(userId, { wishlist });
	}

	// Category methods
	async getCategories() {
		return await this.pb.collection('categories').getFullList();
	}

	// Review methods
	async addProductReview(productId: string, rating: number, comment: string) {
		return await this.pb.collection('reviews').create({
			productId,
			rating,
			comment,
			userId: this.pb.authStore.model?.id
		});
	}

	async getProductReviews(productId: string, page: number = 1, perPage: number = 20) {
		return await this.pb.collection('reviews').getList(page, perPage, {
			filter: `productId = "${productId}"`,
			sort: '-created'
		});
	}

	// Admin methods (requires admin privileges)
	async createProduct(productData: Partial<Product>) {
		return await this.pb.collection('products').create(productData);
	}

	async updateProduct(id: string, productData: Partial<Product>) {
		return await this.pb.collection('products').update(id, productData);
	}

	async deleteProduct(id: string) {
		return await this.pb.collection('products').delete(id);
	}

	async updateStock(productId: string, quantity: number) {
		const product = await this.getProduct(productId);
		return await this.updateProduct(productId, {
			stock: product.stock + quantity
		});
	}
}

export default WebshopAPI;
